package xyz.mtfos.btdemo

import android.Manifest
import android.annotation.SuppressLint
import android.app.Activity
import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import android.os.Handler
import android.widget.*
import xyz.mtfos.btdemo.objectTool.bind

/**
 * Created by jay on 2017/8/9.
 */
class MainActivity : Activity() {
    val mainLay: LinearLayout by bind(R.id.mainlay)

    val intxt: EditText by bind(R.id.intxt)
    val btn: Button by bind(R.id.bt)
    val connectBtn: Button by bind(R.id.connect)
    val disconnectBtn: Button by bind(R.id.disconnect)
    var ble: PrinterBle? = null
    val alignSpinner: Spinner by bind(R.id.align)
    val sizeSpinner: Spinner by bind(R.id.size)
    val alignBtn: Button by bind(R.id.btn_align)
    val sizeBtn: Button by bind(R.id.btn_size)

    val addHR: Button by bind(R.id.addHR)
    val addNL: Button by bind(R.id.addNL)
    val addTAB: Button by bind(R.id.addTAB)
    val addTXT: Button by bind(R.id.addTXT)

    val preview: TextView by bind(R.id.preview)

    val uiHandler: Handler = Handler()
    var th: Thread? = null
    var thrun: Boolean = false
    var isInit: Boolean = false
    val alignArr: Array<String> = arrayOf("Left", "Center", "Right")
    val sizeArr: Array<String> = arrayOf("1,1", "1,2", "2,1", "2,2")

    @SuppressLint("WrongConstant")
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        mainLay.visibility = LinearLayout.GONE

        btn.isEnabled = false
        connectBtn.isEnabled = true
        disconnectBtn.isEnabled = false

        ble = PrinterBle(this@MainActivity, "43:43:A1:12:1F:AC", "dd535b71-8f05-4e30-beb0-6fa7ea3dfc3e", "00000001-8f05-4e30-beb0-6fa7ea3dfc3e")

        ble?.setOnStateChangeListener {
            System.out.println("Now State is >>>> " + it)
            if (it == PrinterBle.CONNECTING) {
                disconnectBtn.isEnabled = true
            }
            if (it == PrinterBle.CONNECTED) {
                btn.isEnabled = true
                connectBtn.isEnabled = false
                disconnectBtn.isEnabled = true
                uiHandler.post {
                    mainLay.visibility = LinearLayout.VISIBLE
                }
            }
            if (it == PrinterBle.DISCONNECTED) {
                btn.isEnabled = false
                connectBtn.isEnabled = true
                disconnectBtn.isEnabled = false
                uiHandler.post {
                    mainLay.visibility = LinearLayout.GONE
                }
            }
        }

        ble?.setOnNotifyListener {
            System.out.println("Get Notify :::::: " + it)
        }

        alignSpinner.adapter = ArrayAdapter<String>(this@MainActivity, android.R.layout.simple_spinner_item, alignArr)
        alignBtn.setOnClickListener {
            val pos = alignSpinner.selectedItemPosition
            when (pos) {
                0 -> ble?.setAlign(PrinterBle.ALIGN_LEFT)
                1 -> ble?.setAlign(PrinterBle.ALIGN_CENTER)
                2 -> ble?.setAlign(PrinterBle.ALIGN_RIGHT)
            }
            refreshPreview()
        }

        sizeSpinner.adapter = ArrayAdapter<String>(this@MainActivity, android.R.layout.simple_spinner_item, sizeArr)
        sizeBtn.setOnClickListener {
            val pos = sizeSpinner.selectedItemPosition
            when (pos) {
                0 -> ble?.setSize(1, 1)
                1 -> ble?.setSize(1, 2)
                2 -> ble?.setSize(2, 1)
                3 -> ble?.setSize(2, 2)
            }
            refreshPreview()
        }

        var permissionCheck: Int = 0
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            permissionCheck += this@MainActivity.checkSelfPermission(android.Manifest.permission.ACCESS_COARSE_LOCATION)
            permissionCheck += this@MainActivity.checkSelfPermission(android.Manifest.permission.ACCESS_FINE_LOCATION)
            if (permissionCheck != 0) {
                requestPermissions(arrayOf(Manifest.permission.ACCESS_COARSE_LOCATION, Manifest.permission.ACCESS_FINE_LOCATION), 111)
            }
        }

        addHR.setOnClickListener {
            ble?.addHR()
            refreshPreview()
        }

        addNL.setOnClickListener {
            ble?.textNewLine()
            refreshPreview()
        }

        addTAB.setOnClickListener {
            ble?.textTab()
            refreshPreview()
        }

        addTXT.setOnClickListener {
            val txt: String = intxt.text.toString()
            ble?.addText(txt)
            refreshPreview()
        }

        btn.setOnClickListener {
            ble?.sendQueue()
        }

        connectBtn.setOnClickListener {
            scanConnect()
        }
        disconnectBtn.setOnClickListener {
            disconnect()
        }
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>?, grantResults: IntArray?) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
    }

    override fun onStop() {
        super.onStop()
        disconnect()
    }

    fun scanConnect() {
        if (ble?.init()!!) {
            ble?.startScan()
        }
    }

    fun disconnect() {
        if (ble?.mState!! == PrinterBle.SCANNING) {
            ble?.stopScan()
        }
        if (ble?.mState!! == PrinterBle.CONNECTED) {
            ble?.disconnect()
        }
    }

    fun refreshPreview() {
        val data = ble?.dataQueue ?: return
        preview.text = ""
        for (str: String in data) {
            preview.text = preview.text.toString() + "\n" + str
        }
    }
}